#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 28.04.2018 19:06
:Licence MIT
Part of grammpy

"""

from .GrammpyException import GrammpyException

class ParsingException(GrammpyException):
    """
    Represent error with parsing
    """
    pass